/*
===============================================================================
 Name        : main.c
 Author      : T.Nishiyama
 Version     : 1.00
 Description : Sample source for W-LAN control system
===============================================================================
*/

#ifdef __USE_CMSIS
#include "LPC122x.h"
#endif

#include <cr_section_macros.h>
#include <NXP/crp.h>
#include <stdio.h>
#include <string.h>

// Variable to store CRP value in. Will be placed automatically
// by the linker when "Enable Code Read Protect" selected.
// See crp.h header for more information
__CRP const unsigned int CRP_WORD = CRP_NO_CRP ;

char     rxdata[32];    // UART0 Rx data
char     txdata[32];    // UART0 Tx data
int      rxcount = 0;   // UART0 Rx count
int16_t  temp10 = 0;    // Temperature * 10

void lpc_init() {

    uint32_t    period;        // timer period
    int         i;

    //// setting for clock/power
    // Enable clock to GPIO0/ADC/UART0
    LPC_SYSCON->SYSAHBCLKCTRL |= ((1 << 31)|(1 << 14)|(1 << 12));
    // Enable Power To ADC
    LPC_SYSCON->PDRUNCFG &= ~(1 << 4);
    // Set ADC CLKDIV to 1MHz
    i = ((SystemCoreClock/1000000)-1)&0xFF;
    LPC_ADC->CR &= (~0x0000FF00);
    LPC_ADC->CR |= ( i << 8 );

    //// setting for PIO
    // no pull up PIO0_7/PIO0_22/PIO0_23/PIO0_24/PIO0_28/PIO0_29/PIO0_30
    LPC_IOCON->PIO0_7 &= (~(1 << 4));
    LPC_IOCON->PIO0_22 &= (~(1 << 4));
    LPC_IOCON->PIO0_23 &= (~(1 << 4));
    LPC_IOCON->PIO0_24 &= (~(1 << 4));
    LPC_IOCON->PIO0_28 &= (~(1 << 4));
    LPC_IOCON->PIO0_29 &= (~(1 << 4));
    LPC_IOCON->R_PIO0_30 &= (~(1 << 4));
    // Set PIO0_30 to AD0 input
    LPC_IOCON->R_PIO0_30 &= ~0x07;
    LPC_IOCON->R_PIO0_30 |= 0x03;
    // Set PIO0_7/PIO0_22/PIO0_23/PIO0_24/PIO0_28/PIO0_29 to output
    LPC_GPIO0->DIR |= 0x31C00080;
    // set PIO0_22/PIO0_29 (default)
    LPC_GPIO0->SET |= ((1 << 22)|(1 << 29));
    // Set PIO0_1 to RXD input.
    LPC_IOCON->PIO0_1 &= ~0x07;
    LPC_IOCON->PIO0_1 |= 0x02;
    // Set PIO0_2 to TXD output.
    LPC_IOCON->PIO0_2 &= ~0x07;
    LPC_IOCON->PIO0_2 |= 0x02;

    // Configure System tick timer in 10msec period
    period = SystemCoreClock / 100; // Period for 10msec SYSTICK
    SysTick_Config(period); // Configuration
}

void uart_init(uint32_t baudrate) {

    uint32_t Fdiv;
    uint32_t regval;

    // Set UART0 CLKDIV to 1
    LPC_SYSCON->UART0CLKDIV = 0x01;

    // Configure UART module
    LPC_UART0->LCR = 0x83;             // 8 bits, no Parity, 1 Stop bit
    regval = LPC_SYSCON->UART0CLKDIV;
    Fdiv = ((SystemCoreClock/regval)/16)/baudrate ;     // baud rate
    LPC_UART0->DLM = Fdiv / 256;
    LPC_UART0->DLL = Fdiv % 256;
    LPC_UART0->LCR = 0x03;      // DLAB = 0
    LPC_UART0->FCR = 0x07;      // Enable and reset TX and RX FIFO. */

    // Clear line status
    regval = LPC_UART0->LSR;

    // Enable UART Interrupt
    NVIC_EnableIRQ(UART0_IRQn);
    LPC_UART0->IER = 0x01;      // Enable Receive Data Available interrupt */
}

void uart_send(char *BufferPtr) {

    while (*BufferPtr) {
        while ( !(LPC_UART0->LSR & (0x01<<5)) );    // wait Transmitter Holding Register Empty
        LPC_UART0->THR = *BufferPtr++;
    }

    return;
}

char uart_recv() {
    char data;

    data = LPC_UART0->RBR;      // get receive data

    return data;
}

int asc2bin(unsigned char c1, unsigned char c2, unsigned char* val)
{
    if( c1 >= 0x30 && c1 <= 0x39)
        c1 = c1 - 0x30;
    else if( c1 >= 0x41 && c1 <= 0x46)
        c1 = c1 - 0x37;
    else
        return -1;

    if( c2 >= 0x30 && c2 <= 0x39)
        c2 = c2 - 0x30;
    else if( c2 >= 0x41 && c2 <= 0x46)
        c2 = c2 - 0x37;
    else
        return -1;

    *val = (unsigned char)(((c1 & 0x0F) << 4) | (c2 & 0x0F));
    return 0;
}

int main(void) {
    
    unsigned char      ledset;        // LED set value

    // initialize LPC1227 
    lpc_init();

    // initialize UART0
    uart_init(115200);  // baud rate = 115200

    // Enter an infinite loop
    while(1) { // Infinite loop

        if (rxcount && rxdata[rxcount-1] == 0x0D)   {   // EOF
            // get temp xx command
            if (strncmp(rxdata, "get temp", 8) == 0) {
                sprintf(txdata, "\r\ntemp = %d.%d degC\r\n", temp10/10, temp10%10);
                uart_send(txdata);

            // set led xx command (x=0-F)
            } else if (strncmp(rxdata, "set led", 7) == 0) {

                if (asc2bin(rxdata[rxcount-3], rxdata[rxcount-2], &ledset) == 0) {    // check x = 0-F
                    // Set LED
                    LPC_GPIO0->SET = ((ledset & 0x18) << 25) | ((ledset & 0x07) << 22);
                    // Clear LED
                    LPC_GPIO0->CLR = ((~ledset & 0x18) << 25) | ((~ledset & 0x07) << 22);

                    uart_send("\r\nset led OK\r\n");
                } else {
                    uart_send("\r\nset number is illegal\r\n");
                }

            // illegal command
            } else {
                    uart_send("\r\nillegal command\r\n");
            }

            rxcount = 0;
        }
    }

    return 0;
}

void SysTick_Handler(void) {
    int16_t  cur_temp;          // Current Temperature
    int16_t  ADCReadVal = 0;    // ADC read value
    uint32_t regval;

    // temparature data from BH1020HFV
    LPC_ADC->CR |= (0x01000000);    // ADC Start

    while(1) {
        regval = LPC_ADC->DR0;
        if ( regval & 0x80000000 ) {    // ADC Done check
            break;
        }
    }

    ADCReadVal = ( regval >> 6 ) & 0x3FF;   // get ADC data

    // convert ADC data to temparature
    // BD1020HFV 30degC = 1.3V -8.2mV/degC
    // 30 + ( 1.3 - ADCReadVal * 3.3 / 2^10 / 0.0082 )
    cur_temp = 188.54 - (long)ADCReadVal * 0.393;   // current temparature
    temp10 = cur_temp + temp10 * 0.9;               // moving average (10)
}

void UART0_IRQHandler(void) {

    rxdata[rxcount++] = uart_recv();

}
